@@ -115,18 +115,19 @@ def _play_time(drive, play, next_play):
     return None
 
 
-def _next_play_with_time(plays, play):
+def _next_play_with(plays, play, pred):
     """
-    Returns the next `nfldb.Play` after `play` in `plays` with valid
-    time data. If such a play does not exist, then `None` is returned.
+    Returns the next `nfldb.Play` after `play` in `plays` where `pred`
+    returns True (given a `nfldb.Play` object).  If such a play does
+    not exist, then `None` is returned.
     """
     get_next = False
     for p in plays:
         if get_next:
-            # Don't take a play without time info.
-            # e.g., Two timeouts in a row, or a two-minute warning
+            # Don't take a play that isn't satisfied.
+            # e.g. for time, Two timeouts in a row, or a two-minute warning
             # next to a timeout.
-            if not p.time:
+            if not pred(p):
                 continue
             return p
         if p.play_id == play.play_id:
@@ -814,6 +815,31 @@ class Clock (object):
         quarter in the game.
         """
 
+    def add_seconds(self, seconds):
+        """
+        Adds the number of seconds given to the current clock time
+        and returns a new clock time. `seconds` may be positive
+        or negative. If a boundary is reached (e.g., `Pregame` or
+        `Final`), then subtracting or adding more seconds has no
+        effect.
+        """
+        elapsed = self.elapsed + seconds
+        phase_jump = 0
+        if elapsed < 0 or elapsed > Clock._phase_max:
+            phase_jump = elapsed // Clock._phase_max
+        try:
+            phase_val = self.phase.value + phase_jump
+            if self.phase.value <= Enums.game_phase.Half.value <= phase_val:
+                phase_val += 1
+            elif phase_val <= Enums.game_phase.Half.value <= self.phase.value:
+                phase_val -= 1
+            phase = Enums.game_phase(phase_val)
+            return Clock(phase, elapsed % (1 + Clock._phase_max))
+        except ValueError:
+            if phase_val < 0:
+                return Clock(Enums.game_phase.Pregame, 0)
+            return Clock(Enums.game_phase.Final, 0)
+
     @property
     def minutes(self):
         """
@@ -1315,6 +1341,19 @@ class PlayPlayer (object):
                 return pval
         return 0.0
 
+    @property
+    def scoring_team(self):
+        """
+        If this is a scoring statistic, returns the team that scored.
+        Otherwise, returns None.
+
+        N.B. `nfldb.PlayPlayer.scoring_team` returns a valid team if
+        and only if `nfldb.PlayPlayer.points` is greater than 0.
+        """
+        if self.points > 0:
+            return self.team
+        return None
+
     @property
     def _row(self):
         return _as_row(PlayPlayer._sql_columns, self)
@@ -1658,6 +1697,44 @@ class Play (object):
                 return pts
         return 0
 
+    @property
+    def scoring_team(self):
+        """
+        If this is a scoring play, returns the team that scored points.
+        Otherwise, returns None.
+
+        N.B. `nfldb.Play.scoring_team` returns a valid team if and only
+        if `nfldb.Play.points` is greater than 0.
+        """
+        for pp in self.play_players:
+            t = pp.scoring_team
+            if t is not None:
+                return t
+        return None
+
+    def score(self, before=False):
+        """
+        Returns the score of the game immediately after this play as a
+        tuple of the form `(home_score, away_score)`.
+
+        If `before` is `True`, then the score will *not* include this
+        play.
+        """
+        game = Game.from_id(self._db, self.gsis_id)
+        if not before:
+            return game.score_at_time(self.time.add_seconds(1))
+
+        s = game.score_at_time(self.time)
+        # The heuristic in `nfldb.Game.score_in_plays` blends TDs and XPs
+        # into a single play (with respect to scoring). So we have to undo
+        # that if we want the score of the game after a TD but before an XP.
+        if self.kicking_xpmade == 1:
+            score_team = self.scoring_team
+            if score_team == game.home_team:
+                return (s[0] - 1, s[1])
+            return (s[0], s[1] - 1)
+        return s
+
     @property
     def _row(self):
         return _as_row(Play._sql_columns, self)
@@ -1764,8 +1841,8 @@ class Drive (object):
         drive._plays = []
         for play in candidates:
             if play.time is None:
-                play.time = _play_time(drive, play,
-                                       _next_play_with_time(candidates, play))
+                next = _next_play_with(candidates, play, lambda p: p.time)
+                play.time = _play_time(drive, play, next)
             if play.time is not None:
                 drive._plays.append(play)
         return drive
@@ -1890,7 +1967,7 @@ class Drive (object):
         game is retrieved from the database if it hasn't been already.
         """
         if self._game is None:
-            return Game.from_id(self.gsis_id)
+            return Game.from_id(self._db, self.gsis_id)
         return self._game
 
     @property
@@ -1917,6 +1994,20 @@ class Drive (object):
                     self._plays.append(p)
         return self._plays
 
+    def score(self, before=False):
+        """
+        Returns the score of the game immediately after this drive as a
+        tuple of the form `(home_score, away_score)`.
+
+        If `before` is `True`, then the score will *not* include this
+        drive.
+        """
+        game = Game.from_id(self._db, self.gsis_id)
+        if before:
+            return game.score_at_time(self.start_time)
+        else:
+            return game.score_at_time(self.end_time)
+
     @property
     def play_players(self):
         """
@@ -1984,7 +2075,7 @@ class Game (object):
 
     _sql_fields = _sql_columns + _sql_derived
 
-    __slots__ = _sql_fields + ['_db', '_drives']
+    __slots__ = _sql_fields + ['_db', '_drives', '_plays']
 
     # Document instance variables for derived SQL fields.
     __pdoc__['Game.winner'] = '''The winner of this game.'''
@@ -2113,6 +2204,7 @@ class Game (object):
         `nfldb.Game.from_row` or `nfldb.Game.from_id`.
         """
         self._drives = None
+        self._plays = None
 
         self._db = db
         """
@@ -2209,6 +2301,8 @@ class Game (object):
         self.time_updated = time_updated
         """The date and time that this game was last updated."""
 
+        self.winner, self.loser = winner, loser
+
     @property
     def is_playing(self):
         """
@@ -2248,11 +2342,106 @@ class Game (object):
         A list of `nfldb.Play` objects in this game. Data is retrieved
         from the database if it hasn't been already.
         """
-        plays = []
-        for drive in self.drives:
-            for play in drive.plays:
-                plays.append(play)
-        return plays
+        if self._plays is None:
+            self._plays = []
+            with Tx(self._db) as cursor:
+                cursor.execute('''
+                    SELECT %s FROM play WHERE gsis_id = %s
+                    ORDER BY time ASC, play_id ASC
+                ''' % (select_columns(Play), '%s'), (self.gsis_id,))
+                for row in cursor.fetchall():
+                    p = Play.from_row(self._db, row)
+                    self._plays.append(p)
+        return self._plays
+
+    def plays_range(self, start, end):
+        """
+        Returns a list of `nfldb.Play` objects for this game in the
+        time range specified. The range corresponds to a half-open
+        interval, i.e., `[start, end)`. Namely, all plays starting at
+        or after `start` up to plays starting *before* `end`.
+
+        The plays are returned in the order in which they occurred.
+
+        `time{1,2}` should be an instance of the `nfldb.Clock` class.
+        (Hint: Values can be created with the `nfldb.Clock.from_str`
+        function.)
+        """
+        import nfldb.query as query
+
+        q = query.Query(self._db)
+        q.play(gsis_id=self.gsis_id, time__ge=start, time__lt=end)
+        q.sort([('time', 'asc'), ('play_id', 'asc')])
+        return q.as_plays()
+
+    def score_in_plays(self, plays):
+        """
+        Returns the scores made by the home and away teams from the
+        sequence of plays given. The scores are returned as a `(home,
+        away)` tuple. Note that this method assumes that `plays` is
+        sorted in the order in which the plays occurred.
+        """
+        # This method is a heuristic to compute the total number of points
+        # scored in a set of plays. Naively, this should be a simple summation
+        # of the `points` attribute of each field. However, it seems that
+        # the JSON feed (where this data comes from) heavily biases toward
+        # omitting XPs. Therefore, we attempt to add them. A brief outline
+        # of the heuristic follows.
+        #
+        # In *most* cases, a TD is followed by either an XP attempt or a 2 PTC
+        # attempt by the same team. Therefore, after each TD, we look for the
+        # next play that fits this criteria, while being careful not to find
+        # a play that has already counted toward the score. If no play was
+        # found, then we assume there was an XP attempt and that it was good.
+        # Otherwise, if a play is found matching the given TD, the point total
+        # of that play is added to the score.
+        #
+        # Note that this relies on the property that every TD is paired with
+        # an XP/2PTC with respect to the final score of a game. Namely, when
+        # searching for the XP/2PTC after a TD, it may find a play that came
+        # after a different TD. But this is OK, so long as we never double
+        # count any particular play.
+        def is_twopta(p):
+            return (p.passing_twopta > 0
+                    or p.receiving_twopta > 0
+                    or p.rushing_twopta > 0)
+
+        counted = set()  # don't double count
+        home, away = 0, 0
+        for i, p in enumerate(plays):
+            pts = p.points
+            if pts > 0 and p.play_id not in counted:
+                counted.add(p.play_id)
+
+                if pts == 6:
+                    def after_td(p2):
+                        return (p.pos_team == p2.pos_team
+                                and (p2.kicking_xpa > 0 or is_twopta(p2))
+                                and p2.play_id not in counted)
+
+                    next = _next_play_with(plays, p, after_td)
+                    if next is None:
+                        pts += 1
+                    elif next.play_id not in counted:
+                        pts += next.points
+                        counted.add(next.play_id)
+                if p.scoring_team == self.home_team:
+                    home += pts
+                else:
+                    away += pts
+        return home, away
+
+    def score_at_time(self, time):
+        """
+        Returns the score of the game at the time specified as a
+        `(home, away)` tuple.
+
+        `time` should be an instance of the `nfldb.Clock` class.
+        (Hint: Values can be created with the `nfldb.Clock.from_str`
+        function.)
+        """
+        start = Clock.from_str('Pregame', '0:00')
+        return self.score_in_plays(self.plays_range(start, time))
 
     @property
     def play_players(self):
